#include "SevSeg.h"
SevSeg sevseg;

unsigned long last_detection_time = 0;  // Time of the last magnet detection
unsigned int rpm = 0;                   // Calculated RPM value
float calculated_value = 0;             // Calculated value using the formula y = 0.0007 * rpm^2.3075
int sensorPin = A0;                     // Sensor connected to analog pin A0
int threshold = 512;                    // Threshold value to detect magnetic field
bool magnet_detected = false;

void setup() {
  Serial.begin(9600);
  
  // SevSeg setup
  byte numDigits = 4;
  byte digitPins[] = {10, 11, 12, 13};         // Pins for digits of the display
  byte segmentPins[] = {9, 2, 3, 5, 6, 8, 7, 4};  // Pins for segments of the display
  
  bool resistorsOnSegments = false;     // Resistors are on digit pins
  bool updateWithDelaysIn = true;       // Flag to determine update delays
  byte hardwareConfig = COMMON_CATHODE; // Type of display (Common Cathode)
  
  sevseg.begin(hardwareConfig, numDigits, digitPins, segmentPins, resistorsOnSegments);
  sevseg.setBrightness(90);  // Set brightness level
}

void loop() {
  // Read value from A0
  int sensorValue = analogRead(sensorPin);  

  // If the value is above the threshold, a magnet is detected
  if (sensorValue > threshold && !magnet_detected) {
    magnet_detect();  // Call the magnet detection function
    magnet_detected = true;  // Set flag to prevent multiple detections
  }

  // Reset the flag when the sensorValue goes below the threshold
  if (sensorValue < threshold) {
    magnet_detected = false;
  }

  // Display the calculated value (multiplied by 100) on the 7-segment display
  sevseg.setNumber(calculated_value * 100, 0);  // Display the calculated value multiplied by 100 with no decimal places
  sevseg.refreshDisplay();  // Refresh the display 
}

// Function called when a magnet is detected
void magnet_detect() {                    
  unsigned long current_time = millis();  // Get the current time

  if (last_detection_time > 0) {  // Ensure it's not the first detection
    // Calculate time between two magnet detections
    unsigned long time_between_detections = current_time - last_detection_time;

    // Calculate RPM (60,000 ms per minute divided by the time for one revolution)
    rpm = 60000 / time_between_detections;

    // Calculate the value y = 0.0007 * rpm^2.3075
    calculated_value = 0.0007 * pow(rpm, 2.3075);

    // Print RPM and calculated value to Serial Monitor
    Serial.print("RPM: ");
    Serial.print(rpm);
    Serial.print("\tCalculated Value: ");
    Serial.println(calculated_value);

    // Print the calculated value multiplied by 100 for reference
    Serial.print("Calculated Value * 100: ");
    Serial.println(calculated_value * 100);
  }

  last_detection_time = current_time;  // Update the last detection time
}
